@@ -1,22 +1,22 @@
 import pkg_resources
 import argparse
+from fnmatch import fnmatch
 from itertools import chain
-from typing import Iterable, Dict, Tuple, Set
+from typing import Iterable, Tuple, Set
 from more_itertools import unique_everseen
 from .utilities import (
     get_required_distribution_names,
-    get_installed_distributions,
+    get_distribution,
     install_requirement,
     iter_configuration_file_requirement_strings,
     get_requirement_string_distribution_name,
     normalize_name,
     is_configuration_file,
-    setup_dist_egg_info,
-    is_editable,
 )
 from ..utilities import iter_parse_delimited_values
 
-_STANDARD_LIBRARY_DISTRIBUTION_NAMES: Set[str] = {
+_DO_NOT_PIN_DISTRIBUTION_NAMES: Set[str] = {
+    # standard library
     "importlib-metadata",
     "importlib-resources",
 }
@@ -26,7 +26,7 @@ def get_frozen_requirements(
     requirements: Iterable[str] = (),
     exclude: Iterable[str] = (),
     exclude_recursive: Iterable[str] = (),
-    no_versions: bool = False,
+    no_version: Iterable[str] = (),
 ) -> Tuple[str, ...]:
     """
     Get the (frozen) requirements for one or more specified distributions or
@@ -42,7 +42,7 @@ def get_frozen_requirements(
       Note: Excluding a distribution here excludes all requirements which would
       be identified through recursively.
       those requirements occur elsewhere.
-    - no_versions (bool) = False: Exclude version numbers from the output
+    - no_version ([str]) = (): Exclude version numbers from the output
       (only return distribution names)
     """
     # Separate requirement strings from requirement files
@@ -50,6 +50,10 @@ def get_frozen_requirements(
         requirements = {requirements}
     else:
         requirements = set(requirements)
+    if isinstance(no_version, str):
+        no_version = (no_version,)
+    elif not isinstance(no_version, tuple):
+        no_version = tuple(no_version)
     requirement_files: Set[str] = set(
         filter(is_configuration_file, requirements)
     )
@@ -77,60 +81,40 @@ def get_frozen_requirements(
                     )
                 ),
                 exclude_recursive=set(map(normalize_name, exclude_recursive)),
-                no_versions=no_versions,
+                no_version=no_version,
             ),
             key=lambda name: name.lower(),
         )
     )
 
 
-def _reload_distribution(
-    distribution: pkg_resources.Distribution,
-) -> None:
-    if is_editable(distribution.project_name):
-        setup_dist_egg_info(distribution.location)
-        # Clear the distribution information cache
-        for cached_property_name in (
-            "_pkg_info",
-            "_key",
-            "_parsed_version",
-            "_version",
-            "__dep_map",
-        ):
-            try:
-                delattr(distribution, cached_property_name)
-            except AttributeError:
-                pass
-
-
 def _iter_frozen_requirements(
     requirement_strings: Iterable[str],
     exclude: Set[str],
     exclude_recursive: Set[str],
-    no_versions: bool = False,
+    no_version: Iterable[str] = (),
 ) -> Iterable[str]:
-    if isinstance(requirement_strings, str):
-        requirement_strings = (requirement_strings,)
-    installed_distributions: Dict[
-        str, pkg_resources.Distribution
-    ] = get_installed_distributions()
-
     def get_requirement_string(distribution_name: str) -> str:
-        # Don't pin importlib-metadata, as it is part of the standard
-        # library and we should use the version distributed with
-        # python
-        if distribution_name in _STANDARD_LIBRARY_DISTRIBUTION_NAMES:
+        def distribution_name_matches_pattern(pattern: str) -> bool:
+            return fnmatch(distribution_name, pattern)
+
+        # * Don't pin importlib-metadata, as it is part of the standard
+        #   library so we should use the version distributed with
+        #   python, and...
+        # * Only include the version in the requirement string if
+        #   the package name does not match any patterns provided in the
+        #   `no_version` argument
+        if (distribution_name in _DO_NOT_PIN_DISTRIBUTION_NAMES) or any(
+            map(distribution_name_matches_pattern, no_version)
+        ):
             return distribution_name
-        nonlocal installed_distributions
         distribution: pkg_resources.Distribution
         try:
-            distribution = installed_distributions[distribution_name]
+            distribution = get_distribution(distribution_name)
         except KeyError:
             # If the distribution is missing, install it
-            install_requirement(distribution_name)
-            installed_distributions = get_installed_distributions()
-            distribution = installed_distributions[distribution_name]
-        _reload_distribution(distribution)
+            install_requirement(distribution_name, echo=False)
+            distribution = get_distribution(distribution_name)
         requirement_string: str = str(distribution.as_requirement())
         # Only include dataclasses for python 3.6
         if distribution_name == "dataclasses":
@@ -155,8 +139,8 @@ def _iter_frozen_requirements(
     requirements: Iterable[str] = unique_everseen(
         chain(*map(get_required_distribution_names_, requirement_strings)),
     )
-    if not no_versions:
-        requirements = map(get_requirement_string, requirements)
+
+    requirements = map(get_requirement_string, requirements)
     return requirements
 
 
@@ -164,7 +148,7 @@ def freeze(
     requirements: Iterable[str] = (),
     exclude: Iterable[str] = (),
     exclude_recursive: Iterable[str] = (),
-    no_versions: bool = False,
+    no_version: Iterable[str] = (),
 ) -> None:
     """
     Print the (frozen) requirements for one or more specified requirements or
@@ -180,8 +164,9 @@ def freeze(
       Note: Excluding a distribution here excludes all requirements which would
       be identified through recursively.
       those requirements occur elsewhere.
-    - no_versions (bool) = False: Exclude version numbers from the output
-      (only print distribution names)
+    - no_version ([str]) = (): Exclude version numbers from the output
+      (only print distribution names) for package names matching any of these
+      patterns
     """
     print(
         "\n".join(
@@ -189,7 +174,7 @@ def freeze(
                 requirements=requirements,
                 exclude=exclude,
                 exclude_recursive=exclude_recursive,
-                no_versions=no_versions,
+                no_version=no_version,
             )
         )
     )
@@ -230,11 +215,15 @@ def main() -> None:
     )
     parser.add_argument(
         "-nv",
-        "--no-versions",
-        default=False,
-        const=True,
-        action="store_const",
-        help="Don't include versions (only output distribution names)",
+        "--no-version",
+        type=str,
+        default=[],
+        action="append",
+        help=(
+            "Don't include versions (only output distribution names) "
+            "for packages matching this glob pattern (note: the value must "
+            "be single-quoted if it contain wildcards)"
+        ),
     )
     arguments: argparse.Namespace = parser.parse_args()
     freeze(
@@ -243,7 +232,7 @@ def main() -> None:
         exclude_recursive=tuple(
             iter_parse_delimited_values(arguments.exclude_recursive)
         ),
-        no_versions=arguments.no_versions,
+        no_version=arguments.no_version,
     )
 
 
