@@ -5,7 +5,8 @@ import os
 from collections import deque
 from pipes import quote
 from itertools import chain
-from subprocess import getstatusoutput
+from subprocess import check_output
+from urllib.parse import urlparse, urlunparse, ParseResult, quote as _quote
 from typing import (
     Any,
     Callable,
@@ -27,14 +28,44 @@ __all__: List[str] = [
     "sys_argv_pop",
     "iter_sys_argv_get",
     "sys_argv_get",
+    "update_url_user_password",
 ]
 lru_cache: Callable[..., Any] = functools.lru_cache
-# For backwards compatibility:
-cerberus: Any
-try:
-    from daves_dev_tools import cerberus
-except ImportError:
-    cerberus = None
+
+
+def update_url_user_password(
+    url: str,
+    user: str,
+    password: str = "",
+    quote: Callable[[str], str] = _quote,
+) -> str:
+    """
+    Update a URL's user and password and return the result.
+
+    Parameters:
+
+    - url (str)
+    - user (str)
+    - password (str) = "": (optional)
+    - quote = urllib.parse.quote: A function to use for escaping
+      invalid character (defaults to `urllib.parse.quote`)
+    """
+    assert url and user
+    parse_result: ParseResult = urlparse(url)
+    host: str = parse_result.netloc.rpartition("@")[-1]
+    user_password: str = quote(user)
+    if password:
+        user_password = f"{user_password}:{quote(password)}"
+    return urlunparse(
+        (
+            parse_result.scheme,
+            f"{user_password}@{host}",
+            parse_result.path,
+            parse_result.params,
+            parse_result.query,
+            parse_result.fragment,
+        )
+    )
 
 
 def _iter_parse_delimited_value(value: str, delimiter: str) -> Iterable[str]:
@@ -58,7 +89,7 @@ def iter_parse_delimited_values(
     return chain(*map(iter_parse_delimited_value_, values))
 
 
-def run(command: str, echo: bool = True) -> str:
+def run(command: Sequence[str], echo: bool = True) -> str:
     """
     This function runs a shell command, raises an error if a non-zero
     exit code is returned, and echo's both the command and output *if*
@@ -66,22 +97,25 @@ def run(command: str, echo: bool = True) -> str:
 
     Parameters:
 
-    - command (str): A shell command
+    - command (str|[str]): A shell command
     - echo (bool) = True: If `True`, the command and the output from the
       command will be printed to stdout
     """
     if echo:
-        print(command)
-    status: int
-    output: str
-    status, output = getstatusoutput(command)
-    # Create an error if a non-zero exit status is encountered
-    if status:
-        raise OSError(output if echo else f"$ {command}\n{output}")
-    else:
-        output = output.strip()
-        if output and echo:
-            print(output)
+        command_str: str
+        if isinstance(command, str):
+            command_str = command
+        else:
+            command_str = " ".join(map(quote, command))
+        print(command_str)
+    output: str = check_output(
+        command,
+        encoding="utf-8",
+        universal_newlines=True,
+        shell=isinstance(command, str),
+    ).strip()
+    if echo:
+        print(output)
     return output
 
 
